import json
from django.core.urlresolvers import reverse
from django.test import Client
from unittest import TestCase
from django.test.utils import override_settings


class TestView(TestCase):

    def setUp(self):
        self.client = Client()

    @override_settings(GEOIP_PATH='/wrong/directory/')
    def test_wrong_geoip_path(self):
        """
        Check that settings an invalid path yields a HTTP 500.
        """
        url = reverse('easygeoip:geoip-explicit-ip-view', args=("95.12.13.14",))
        res = self.client.get(url)
        self.assertEqual(res.status_code, 500)

    def test_invalid_ip_address(self):
        """
        Test that giving an invalid IP address yields a HTTP 400.
        """
        url = reverse('easygeoip:geoip-explicit-ip-view', args=("0.0.0.0",))
        res = self.client.get(url)
        self.assertEqual(res.status_code, 400)

    def test_valid_ip_address(self):
        """
        Test that giving a valid IP address yields an object containing specific information.
        """
        example_dot_com_ip = "93.184.216.34"
        url = reverse('easygeoip:geoip-explicit-ip-view', args=(example_dot_com_ip,))
        res = self.client.get(url)
        self.assertEqual(res.status_code, 200)
        self.assertDictEqual(json.loads(res.content), {
            "city": "Norwell",
            "continent_code": "NA",
            "region": "MA",
            "charset": 0,
            "area_code": 781,
            "longitude": -70.82279968261719,
            "country_code3": "USA",
            "latitude": 42.15079879760742,
            "postal_code": "02061",
            "dma_code": 506,
            "country_code": "US",
            "country_name": "United States",
            "ip_address": "93.184.216.34"
        })






import os
import requests
import logging
import gzip
from celery.task import task
from . import settings


GEOIP_BASE_URL = 'http://geolite.maxmind.com/download/geoip/database/'
GEOLITE_CITY_FILENAME = 'GeoLiteCity.dat.gz'
GEOIP_DIR = 'GeoLiteCountry/'
GEOIP_FILENAME = 'GeoIP.dat.gz'


logger = logging.getLogger(__name__)


def extract_gzip(filename):
    """
    Exctract the given gzip infile and write the content into outfile.
    """
    logger.info("Extracting " + filename)
    to_extract = gzip.open(filename, 'rb')
    extracted_filename = os.path.splitext(filename)[0]  # Remove .gz extension
    logger.info("Writing to " + extracted_filename)
    output = open(extracted_filename, 'wb')
    for line in to_extract:
        output.write(line)
    output.close()
    to_extract.close()


def download_file(url, outfile):
    """
    Download the given URL to the specified out_file.
    """
    logger.info("Downloading " + url)
    res = requests.get(url, stream=True)
    if res.status_code == 200:
        output = open(outfile, 'wb')
        chunks = res.iter_content(chunk_size=1024)
        for chunk in chunks:
            if chunk:  # Filter out keep-alive chunks
                output.write(chunk)
        output.close()
        logger.info("Downloaded in " + outfile)
        return True
    else:
        logger.error("Resource " + url + " is not available (status code was " + str(res.status_code))
        return False


@task
def update_geoip_database():
    """
    Download and extract the database files needed for the django.contrib.gis.geoip package to work.
    """
    db_path = settings.get_geoip_path()
    if not os.path.exists(db_path):
        os.makedirs(db_path)

    to_download = [
        {
            'outfile': db_path + GEOLITE_CITY_FILENAME,
            'url': GEOIP_BASE_URL + GEOLITE_CITY_FILENAME,
        },
        {
            'outfile': db_path + GEOIP_FILENAME,
            'url': GEOIP_BASE_URL + GEOIP_DIR + GEOIP_FILENAME
        }

    ]
    # Download and extract binary files
    for dbfile in to_download:
        if download_file(dbfile['url'], dbfile['outfile']):
            extract_gzip(dbfile['outfile'])
            os.remove(dbfile['outfile'])




import os
from django.conf import settings


# Provide necessary defaults

# GeoIP database directory
BASE_DIR = os.path.dirname(os.path.dirname(__file__))
DEFAULT_DATA_DIR = "easygeoip/geoip_data/"


def get_geoip_path():
    """
    We evaluate the path at runtime to be able to override settings in tests.
    """
    return getattr(settings, 'GEOIP_PATH', os.path.join(BASE_DIR, DEFAULT_DATA_DIR))


import json
import logging
from django.contrib.gis.geoip import GeoIP, GeoIPException
from django.http import HttpResponse
from django.views.generic import View
from easygeoip.settings import get_geoip_path

ERRORS = {
    'no_geoip_path': {
        'reason': 'Could not instantiate a GeoIP object. Did you set the GEOIP_PATH variable and downloaded the files?'
    },
    'invalid_ip_address': {
        'reason': 'Invalid IP address'
    }
}


logger = logging.getLogger(__name__)


class LocationFromIpView(View):
    """
    A simple endpoint that allows to retrieve geo-location information for a specific IP address.
    """
    def get(self, *args, **kwargs):
        ip_address = kwargs.get('ip_address')
        # If the app is running behind a proxy we have to check for the X-Forwarded-For header
        if not ip_address:
            ip_address = self.request.META.get('HTTP_X_FORWARDED_FOR', '') or self.request.META.get('REMOTE_ADDR')
        try:
            logger.debug("Using path " + get_geoip_path())
            g = GeoIP(get_geoip_path())
        except GeoIPException as ge:
            return HttpResponse(json.dumps({
                'ip_address': ip_address,
                'reason': ERRORS['no_geoip_path']['reason'],
                'exception_message': ge.message
            }), status=500, content_type='application/json')
        location_info = g.city(ip_address)
        if not location_info:
            return HttpResponse(json.dumps({
                'ip_address': ip_address,
                'reason': ERRORS['invalid_ip_address']['reason']
            }), status=400, content_type='application/json')
        location_info['ip_address'] = ip_address
        return HttpResponse(json.dumps(location_info), content_type='application/json')



from django.conf.urls import patterns, url

# API URLs
from .views import LocationFromIpView

urlpatterns = patterns('',
    url(r'^location/(?P<ip_address>(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}))/$', LocationFromIpView.as_view(),
        name='geoip-explicit-ip-view'),
    url(r'^location/$', LocationFromIpView.as_view(), name='geoip-implicit-ip-view')  # Take IP addr from request
)
